var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
export class DataMigrationService {
    constructor() {
        this.exNativeMig = navigator.exalusMigration;
        try {
            const currentUrl = window.location.href;
            const url = new URL(currentUrl);
            const params = new URLSearchParams(url.search);
            const encMig = params.get('mig_c_enc');
            const migration = params.get('mig_controllers');
            const migLang = params.get('mig_lang');
            if (encMig !== null) {
                const decodedData = decodeURIComponent(this.b64DecodeUnicode(decodeURIComponent(encMig)));
                localStorage.setItem(DataMigrationService.AppUserControllersKeyName, decodedData);
            }
            if (migration !== null) {
                localStorage.setItem(DataMigrationService.AppUserControllersKeyName, decodeURIComponent(migration));
            }
            if (migLang !== null) {
                localStorage.setItem(DataMigrationService.AppSelectedLanguageKeyName, migLang);
            }
        }
        catch (ex) {
            console.error(ex);
        }
    }
    b64DecodeUnicode(str) {
        return decodeURIComponent(Array.prototype.map.call(atob(str), function (c) {
            return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
        }).join(''));
    }
    decodeUriJsonData(data) {
        return JSON.parse(decodeURIComponent(data));
    }
    encodeUriJsonData(data) {
        return encodeURIComponent(JSON.stringify(data));
    }
    IsExalusNativeMigrationAvailable() {
        return navigator.exalusMigration !== undefined;
    }
    GetServiceName() {
        return DataMigrationService.ServiceName;
    }
    IsMigrationDataAvailableAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.IsExalusNativeMigrationAvailable()) {
                return (yield this.exNativeMig.IsMigrationDataAvailableAsync()).DataMigrated;
            }
            else if (localStorage.getItem(DataMigrationService.AppSelectedLanguageKeyName) !== null || localStorage.getItem(DataMigrationService.AppUserControllersKeyName) !== null)
                return true;
            else
                return false;
        });
    }
    GetControllersDataAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.IsExalusNativeMigrationAvailable()) {
                let controllers = yield this.exNativeMig.GetControllersDataAsync();
                return controllers;
            }
            let controllers = localStorage.getItem(DataMigrationService.AppUserControllersKeyName);
            if (controllers !== null) {
                return JSON.parse(controllers);
            }
            else
                return null;
        });
    }
    GetLanguageCodeAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.IsExalusNativeMigrationAvailable()) {
                return yield this.exNativeMig.GetLanguageCodeAsync();
            }
            return localStorage.getItem(DataMigrationService.AppSelectedLanguageKeyName);
        });
    }
}
DataMigrationService.AppSelectedLanguageKeyName = "App___SelectedLanguage";
DataMigrationService.AppUserControllersKeyName = "App___UserControllersListDatabase";
DataMigrationService.ServiceName = "DataMigrationService";
//# sourceMappingURL=DataMigrationService.js.map